@@ -30,6 +30,9 @@ from chgksuite.common import (
 
 from collections import defaultdict
 import json
+import shlex
+
+LANGS = ["ru", "ua", "by", "en", "uz", "uz_cyr", "custom"]
 
 try:
     basestring
@@ -38,7 +41,6 @@ except NameError:
 
 debug = False
 
-
 class VarWrapper(object):
     def __init__(self, name, var):
         self.name = name
@@ -131,7 +133,7 @@ class ParserWrapper(object):
             else:
                 result.append(to_append)
                 result_to_print.append(to_append)
-        print("Command line call: {}".format(" ".join(result_to_print)))
+        print("Command line call: {}".format(shlex.join(result_to_print)))
         return result
 
     def ok_button_press(self):
@@ -208,6 +210,22 @@ class ParserWrapper(object):
             )
             checkbutton.pack(side="left")
             self.vars.append(VarWrapper(name=args[0], var=var))
+        elif argtype == "radiobutton":
+            var = tk.StringVar()
+            var.set(kwargs["default"])
+            innerframe = tk.Frame(frame)
+            innerframe.pack(side="top")
+            label = tk.Label(innerframe, text=caption)
+            label.pack(side="left")
+            for ch in kwargs["choices"]:
+                radio = tk.Radiobutton(
+                    innerframe,
+                    text=ch,
+                    variable=var,
+                    value=ch,
+                )
+                radio.pack(side="left")
+            self.vars.append(VarWrapper(name=args[0], var=var))
         elif argtype in {"filename", "folder"}:
             text = "(имя файла)" if argtype == "filename" else "(имя папки)"
             button_text = "Открыть файл" if argtype == "filename" else "Открыть папку"
@@ -349,6 +367,15 @@ class ArgparseBuilder:
             caption="Имя файла",
             filetypes=[("chgksuite parsable files", ("*.docx", "*.txt"))],
         )
+        self.add_argument(
+            cmdparse,
+            "--language", "-lang",
+            help="language to use while parsing.",
+            choices=LANGS,
+            default="ru",
+            caption="Язык",
+            argtype="radiobutton"
+        )
         self.add_argument(
             cmdparse,
             "--defaultauthor",
@@ -369,7 +396,7 @@ class ArgparseBuilder:
             cmdparse,
             "--regexes",
             default=None,
-            help="A file containing regexes " "(the default is regexes.json).",
+            help="A file containing regexes " "(the default is regexes_ru.json).",
             advanced=True,
             caption="Файл с регулярными выражениями",
             argtype="filename",
@@ -444,6 +471,15 @@ class ArgparseBuilder:
             advanced=True,
             hide=True,
         )
+        self.add_argument(
+            cmdcompose,
+            "--language", "-lang",
+            help="language to use while composing.",
+            choices=LANGS,
+            default="ru",
+            caption="Язык",
+            argtype="radiobutton"
+        )
         self.add_argument(
             cmdcompose,
             "--nots",
@@ -855,17 +891,18 @@ def app():
     if isinstance(sourcedir, bytes):
         sourcedir = sourcedir.decode("utf8")
     ld = get_lastdir()
-    parser = argparse.ArgumentParser(prog="chgksuite".format(__version__))
+    parser = argparse.ArgumentParser(prog="chgksuite")
     use_wrapper = len(sys.argv) == 1 and TKINTER
     if use_wrapper:
         parser = ParserWrapper(parser, lastdir=ld)
     ArgparseBuilder(parser, use_wrapper).build()
     args = DefaultNamespace(parser.parse_args())
 
-    if not args.labels_file:
-        args.labels_file = os.path.join(resourcedir, "labels_ru.toml")
-    if not args.regexes:
-        args.regexes = os.path.join(resourcedir, "regexes.json")
+    if args.language in LANGS:
+        if args.action == "parse":
+            args.regexes = os.path.join(resourcedir, f"regexes_{args.language}.json")
+        elif args.action == "compose":
+            args.labels_file = os.path.join(resourcedir, f"labels_{args.language}.toml")
     if not args.docx_template:
         args.docx_template = os.path.join(resourcedir, "template.docx")
     if not args.pptx_config:
